;; Star theme 0.4
;; Implemented by Stefan Zeiger <szeiger@szeiger.de>

(defgroup Star "Star Theme" :group appearance)

(defcustom Star:inactive-font-color "#c0bfc0bfc0bf"
  "Text color on title bars of inactive windows."
  :type color
  :group (appearance Star)
  :after-set after-setting-frame-option)

(defcustom Star:focused-font-color "#ffffff"
  "Text color on title bars of focused windows."
  :type color
  :group (appearance Star)
  :after-set after-setting-frame-option)

(defcustom Star:transients-can-iconify t
  "Transient windows can have iconify images."
  :type boolean
  :group (appearance Star)
  :after-set after-setting-frame-option)

(defcustom Star:transients-can-max t
  "Transient windows can have maximize/restore images."
  :type boolean
  :group (appearance Star)
  :after-set after-setting-frame-option)

(defcustom Star:show-resize-triangle nil
  "Show a resize triangle in the lower right corner of resizable windows."
  :type boolean
  :group (appearance Star)
  :after-set after-setting-frame-option)

(defcustom Star:icon-mode 'if-available
  "Show window icon in title bar: \\w"
  :type (choice (always       "Always (use default icon where necessary)")
		(if-available "If available (don't use the default icon)")
		(never        "Never"))
  :group (appearance Star)
  :after-set after-setting-frame-option)

(defun Star:draw-tiled-image (src dst x y end)
       (when (<= x end)
         (composite-images dst src x y)
         (Star:draw-tiled-image src dst (+ x 50) y end)
  ))

(let*
    (
     (default-icon (list (make-image "default-icon.png")))

     (menu-image (lambda (w)
                         (let ((icon (window-icon-image w)))
                              (if icon (list icon) default-icon))
                         ))

     (hide-menu-image
       (lambda (w)
         (case Star:icon-mode
           ((always)       nil)
           ((if-available) (let ((icon (window-icon-image w))) (if icon nil t)) )
           ((never)        t)
         )))

     (text-left-edge (lambda (w) (if (hide-menu-image w) 0 14)))

     (hide-transient-iconify (lambda (w) (not Star:transients-can-iconify)))

     (hide-transient-max (lambda (w) (not Star:transients-can-max)))

     (hide-resize-triangle (lambda (w) (not Star:show-resize-triangle)))

     (font-colors (lambda (w) (list Star:inactive-font-color
                                    Star:focused-font-color)))

     (ltitle-colors `("#624ca5" "#9c6ff7"))

     (title-image (list (make-image "inactive-title.png")
                        (make-image "focused-title.png")))

     (title-image-c (list (make-image "inactive-title-cropped.png")
                          (make-image "focused-title-cropped.png")))

     (close-image (list (make-image "inactive-close.png")
                        (make-image "focused-close.png")
                        (make-image "highlighted-close.png")
                        (make-image "clicked-close.png")))

     (iconify-image (list (make-image "inactive-iconify.png")
                          (make-image "focused-iconify.png")
                          (make-image "highlighted-iconify.png")
                          (make-image "clicked-iconify.png")))

     (max-image (list (make-image "inactive-maximize.png")
                      (make-image "focused-maximize.png")
                      (make-image "highlighted-maximize.png")
                      (make-image "clicked-maximize.png")))

     (restore-image (list (make-image "inactive-restore.png")
                          (make-image "focused-restore.png")
                          (make-image "highlighted-restore.png")
                          (make-image "clicked-restore.png")))

     (max-restore-image
        (lambda (w) (if (window-maximized-p w) restore-image max-image)))

     (top-image-left (list (make-image "inactive-tborder.png")
                           (make-image "focused-tborder-1.png")))

     (top-image-right (list (make-image "inactive-tborder.png")
                            (make-image "focused-tborder-2.png")))

     (top-left-image (list (make-image "inactive-tlcorner.png")
                           (make-image "focused-tlcorner.png")))

     (top-right-image (list (make-image "inactive-trcorner.png")
                            (make-image "focused-trcorner.png")))

     (bottom-left-image (list (make-image "inactive-blcorner.png")
                              (make-image "focused-blcorner.png")))

     (bottom-right-image (list (make-image "inactive-brcorner.png")
                               (make-image "focused-brcorner.png")))

     (bottom-image (list (make-image "inactive-bborder.png")
                         (make-image "focused-bborder.png")))

     (left-image (list (make-image "inactive-lborder.png")
                       (make-image "focused-lborder.png")))

     (right-image (list (make-image "inactive-rborder.png")
                        (make-image "focused-rborder.png")))

     (left-shaded-image (list (make-image "inactive-lshaded.png")
                              (make-image "focused-lshaded.png")))

     (right-shaded-image (list (make-image "inactive-rshaded.png")
                               (make-image "focused-rshaded.png")))

     (ltitle-background-image (list (make-image "inactive-title-background.png")
                                    (make-image "focused-title-background-1.png")))

     (rtitle-background-image (list (make-image "inactive-title-background.png")
                                    (make-image "focused-title-background-2.png")))

     (left-gradient (list (make-image "inactive-lborder-gradient.png")
                          (make-image "focused-lborder-gradient.png")))

     (left-shaded-back (list (make-image "inactive-shadedback.png")
                             (make-image "focused-lshadedback.png")))

     (right-shaded-back (list (make-image "inactive-shadedback.png")
                              (make-image "focused-rshadedback.png")))

     (shaded-title-image (list (make-image "inactive-shaded-title.png")
                               (make-image "focused-shaded-title.png")))

     (resize-triangle-image (list (make-image "resize-triangle.png")))

     (render-normal-title (lambda (img state)
       (let*
         (
          (active (or (eq state 'highlighted)
                      (eq state 'focused)
                      (eq state 'clicked)
            ))
          (title-width (car (image-dimensions img)))
          (limg (if active (car (cdr ltitle-background-image))
                           (car ltitle-background-image)))
          (mimg (if active (car (cdr title-image-c))
                           (car title-image-c)))
          (rimg (if active (car (cdr rtitle-background-image))
                           (car rtitle-background-image)))
          (mimg-start (- title-width 79))
          (rimg-start (- title-width 54))
         )
         (when (> title-width 0)
           (if (> mimg-start 0)
             (progn
               (Star:draw-tiled-image limg img 0 1 (- mimg-start 1))
               (Star:draw-tiled-image rimg img rimg-start 1
                                                     (- title-width 1))
               (composite-images img mimg mimg-start 1)
             )
             (progn
               (Star:draw-tiled-image rimg img 0 1
                                                     (- title-width 1))
               (composite-images img mimg 0 1)
             )
           )
         )
     )))

     (render-shaded-title (lambda (img state)
       (let*
         (
          (active (or (eq state 'highlighted)
                      (eq state 'focused)
                      (eq state 'clicked)
            ))
          (title-width (car (image-dimensions img)))
          (limg (if active (car (cdr left-shaded-back))
                           (car left-shaded-back)))
          (mimg (if active (car (cdr shaded-title-image))
                           (car shaded-title-image)))
          (rimg (if active (car (cdr right-shaded-back))
                           (car right-shaded-back)))
          (mimg-start (- title-width 79))
          (rimg-start (- title-width 54))
         )
         (when (> title-width 0)
           (if (> mimg-start 0)
             (progn
               (Star:draw-tiled-image limg img 0 0 (- mimg-start 1))
               (Star:draw-tiled-image rimg img rimg-start 0
                                                     (- title-width 1))
               (composite-images img mimg mimg-start 0)
             )
             (progn
               (Star:draw-tiled-image rimg img 0 0
                                                     (- title-width 1))
               (composite-images img mimg 0 0)
             )
           )
         )
     )))

     (frame `(
       ((right-edge . 56)
        (top-edge . -20)
        (height . 20)
        (cursor . left_ptr)
        (foreground . ,font-colors)
        (left-edge . ,text-left-edge)
        (y-justify . 2)
        (class . title)
        (text . ,window-name)
        (renderer . ,render-normal-title)
        (x-justify . 8))

       ((left-edge . 0)
        (width . 14)
        (top-edge . -20)
        (height . 20)
        (cursor . left_ptr)
        (class . title)
        (hidden . ,hide-menu-image)
        (background . ,ltitle-background-image))

       ((cursor . left_ptr)
        (right-edge . 0)
        (top-edge . -20)
        (width . 56)
        (height . 20)
        (class . title)
        (background . ,rtitle-background-image))

       ((width . 16)
        (top-edge . -19)
        (cursor . left_ptr)
        (height . 16)
        (class . menu-button)
        (left-edge . 0)
        (scale-foreground . t)
        (hidden . ,hide-menu-image)
        (foreground . ,menu-image)
        (background . ,ltitle-colors))

       ((cursor . left_ptr)
        (right-edge . 0)
        (top-edge . -19)
        (width . 17)
        (removable . t)
        (foreground . ,close-image)
        (class . close-button)
        (background . ,rtitle-background-image))

       ((right-edge . 18)
        (cursor . left_ptr)
        (foreground . ,max-restore-image)
        (removable . t)
        (width . 17)
        (top-edge . -19)
        (class . maximize-button)
        (background . ,rtitle-background-image))

       ((right-edge . 36)
        (cursor . left_ptr)
        (width . 17)
        (foreground . ,iconify-image)
        (removable . t)
        (top-edge . -19)
        (class . iconify-button)
        (background . ,rtitle-background-image))

       ((right-edge . 135)
        (left-edge . 0)
        (top-edge . -22)
        (class . top-border)
        (background . ,top-image-left))

       ((left-edge . -4)
        (class . top-left-corner)
        (top-edge . -22)
        (background . ,top-left-image))

       ((right-edge . -4)
        (class . top-right-corner)
        (top-edge . -22)
        (background . ,top-right-image))

       ((left-edge . -4)
        (top-edge . 30)
        (class . left-border)
        (bottom-edge . 0)
        (background . ,left-image))

       ((class . right-border)
        (right-edge . -4)
        (top-edge . 0)
        (bottom-edge . 0)
        (background . ,right-image))

       ((class . bottom-border)
        (left-edge . 0)
        (right-edge . 0)
        (bottom-edge . -4)
        (background . ,bottom-image))

       ((class . bottom-left-corner)
        (left-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-left-image))

       ((class . bottom-right-corner)
        (right-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-right-image))

       ((class . bottom-right-corner)
        (right-edge . 0)
        (bottom-edge . 0)
        (hidden . ,hide-resize-triangle)
        (background . ,resize-triangle-image))

       ((top-edge . -22)
        (right-edge . 0)
        (width . 110)
        (background . ,top-image-right)
        (class . top-border))

       ((right-edge . 110)
        (height . 3)
        (top-edge . -22)
        (background . ,title-image)
        (foreground . ,title-image)
        (class . top-border))

       ((background . ,left-gradient)
        (height . 30)
        (top-edge . 0)
        (left-edge . -4)
        (class . left-border))))

     (shaped-frame `(
       ((height . 22)
        (cursor . left_ptr)
        (top-edge . -22)
        (right-edge . 56)
        (renderer . ,render-shaded-title)
        (class . title)
        (text . ,window-name)
        (x-justify . 8)
        (y-justify . 4)
        (left-edge . ,text-left-edge)
        (foreground . ,font-colors))

       ((height . 22)
        (width . 14)
        (cursor . left_ptr)
        (top-edge . -22)
        (left-edge . 0)
        (background . ,left-shaded-back)
        (class . title)
        (hidden . ,hide-menu-image))

       ((cursor . left_ptr)
        (right-edge . 0)
        (width . 56)
        (top-edge . -22)
        (height . 22)
        (class . title)
        (background . ,right-shaded-back))

       ((width . 16)
        (height . 16)
        (top-edge . -19)
        (cursor . left_ptr)
        (left-edge . 0)
        (class . menu-button)
        (scale-foreground . t)
        (hidden . ,hide-menu-image)
        (foreground . ,menu-image)
        (background . ,ltitle-colors))

       ((right-edge . 0)
        (cursor . left_ptr)
        (width . 17)
        (height . 18)
        (top-edge . -19)
        (removable . t)
        (foreground . ,close-image)
        (class . close-button)
        (background . ,right-shaded-back))

       ((cursor . left_ptr)
        (right-edge . 18)
        (removable . t)
        (top-edge . -19)
        (class . maximize-button)
        (background . ,max-restore-image))

       ((cursor . left_ptr)
        (right-edge . 36)
        (width . 17)
        (removable . t)
        (top-edge . -19)
        (class . iconify-button)
        (background . ,iconify-image))

       ((cursor . left_ptr)
        (left-edge . -4)
        (top-edge . -22)
        (class . title)
        (background . ,left-shaded-image))

       ((cursor . left_ptr)
        (right-edge . -4)
        (class . title)
        (top-edge . -22)
        (background . ,right-shaded-image))))

     (transient-frame `(
       ((right-edge . 56)
        (top-edge . -20)
        (height . 20)
        (cursor . left_ptr)
        (foreground . ,font-colors)
        (left-edge . 0)
        (y-justify . 2)
        (class . title)
        (text . ,window-name)
        (renderer . ,render-normal-title)
        (x-justify . 8))

       ((cursor . left_ptr)
        (right-edge . 0)
        (top-edge . -20)
        (width . 56)
        (height . 20)
        (class . title)
        (background . ,rtitle-background-image))

       ((cursor . left_ptr)
        (right-edge . 0)
        (top-edge . -19)
        (width . 17)
        (removable . t)
        (foreground . ,close-image)
        (class . close-button)
        (background . ,rtitle-background-image))

       ((right-edge . 18)
        (cursor . left_ptr)
        (foreground . ,max-restore-image)
        (removable . t)
        (width . 17)
        (top-edge . -19)
        (hidden . ,hide-transient-max)
        (class . maximize-button)
        (background . ,rtitle-background-image))

       ((right-edge . 36)
        (cursor . left_ptr)
        (width . 17)
        (foreground . ,iconify-image)
        (hidden . ,hide-transient-iconify)
        (removable . t)
        (top-edge . -19)
        (class . iconify-button)
        (background . ,rtitle-background-image))

       ((right-edge . 135)
        (left-edge . 0)
        (top-edge . -22)
        (class . top-border)
        (background . ,top-image-left))

       ((left-edge . -4)
        (class . top-left-corner)
        (top-edge . -22)
        (background . ,top-left-image))

       ((right-edge . -4)
        (class . top-right-corner)
        (top-edge . -22)
        (background . ,top-right-image))

       ((left-edge . -4)
        (top-edge . 30)
        (class . left-border)
        (bottom-edge . 0)
        (background . ,left-image))

       ((class . right-border)
        (right-edge . -4)
        (top-edge . 0)
        (bottom-edge . 0)
        (background . ,right-image))

       ((class . bottom-border)
        (left-edge . 0)
        (right-edge . 0)
        (bottom-edge . -4)
        (background . ,bottom-image))

       ((class . bottom-left-corner)
        (left-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-left-image))

       ((class . bottom-right-corner)
        (right-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-right-image))

       ((top-edge . -22)
        (right-edge . 0)
        (width . 110)
        (background . ,top-image-right)
        (class . top-border))

       ((right-edge . 110)
        (height . 3)
        (top-edge . -22)
        (background . ,title-image)
        (foreground . ,title-image)
        (class . top-border))

       ((background . ,left-gradient)
        (height . 30)
        (top-edge . 0)
        (left-edge . -4)
        (class . left-border))))

     (shaped-transient-frame `(
       ((height . 22)
        (cursor . left_ptr)
        (top-edge . -22)
        (right-edge . 56)
        (renderer . ,render-shaded-title)
        (class . title)
        (text . ,window-name)
        (x-justify . 8)
        (y-justify . 4)
        (left-edge . 0)
        (foreground . ,font-colors))

       ((cursor . left_ptr)
        (right-edge . 0)
        (width . 56)
        (top-edge . -22)
        (height . 22)
        (class . title)
        (background . ,right-shaded-back))

       ((right-edge . 0)
        (cursor . left_ptr)
        (width . 17)
        (height . 18)
        (top-edge . -19)
        (removable . t)
        (foreground . ,close-image)
        (class . close-button)
        (background . ,right-shaded-back))

       ((cursor . left_ptr)
        (right-edge . 18)
        (removable . t)
        (top-edge . -19)
        (hidden . ,hide-transient-max)
        (class . maximize-button)
        (background . ,max-restore-image))

       ((cursor . left_ptr)
        (right-edge . 36)
        (width . 17)
        (removable . t)
        (top-edge . -19)
        (hidden . ,hide-transient-iconify)
        (class . iconify-button)
        (background . ,iconify-image))

       ((cursor . left_ptr)
        (left-edge . -4)
        (top-edge . -22)
        (class . title)
        (background . ,left-shaded-image))

       ((cursor . left_ptr)
        (right-edge . -4)
        (class . title)
        (top-edge . -22)
        (background . ,right-shaded-image))))

    )

    (add-frame-style 'Star
                     (lambda (w type)
                       (cond ((eq type 'shaped)           shaped-frame)
                             ((eq type 'shaded)           shaped-frame)
                             ((eq type 'transient)        transient-frame)
                             ((eq type 'shaped-transient) shaped-transient-frame)
                             ((eq type 'shaded-transient) shaped-transient-frame)
                             ((eq type 'unframed)         nil-frame)
                             (t                           frame))))
)
